--[[
    BGLive is an user interface for spectated battleground 
	wargames in World of Warcraft.
    Copyright (C) 2015  Harald Böhm <harald@boehm.agency>
	Further contributors: Jochen Taeschner and Romina Schmidt.
	
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

-- ArenaLive addon Name and localisation table:
local addonName, L = ...;

--[[
**************************************************
******* GENERAL HANDLER SET UP STARTS HERE *******
**************************************************
]]--
local BaseIndicator = ArenaLive:ConstructHandler("BaseIndicator", true);
local Cooldown = ArenaLive:GetHandler("Cooldown");

BaseIndicator:RegisterEvent("UPDATE_BATTLEFIELD_STATUS");

local baseIndicators = {};

local BaseIndicatorClass = {};

local NUM_MAX_BASES = 7;
local SEC_TO_TURN_BASE = 60;
local BASE_ICON_HEIGHT = 32;
local BASE_ICON_WIDTH = 32;

--[[
	TextureID Reference:
	Stables Icon: 36 - 40
	Gold Mine Icon: 16 - 20
	Blacksmith Icon: 26 - 30
	Lumber Mill Icon: 21 - 25
	Farm Icon: 31 - 35
	Tower Icon: 6, 9 - 12
	Workshop: 135 - 139
	Hangar: 140 - 144
	Docks: 145 - 149
	Refinery: 150 - 154
]]
local baseStates = {
	["neutral"] = {
		[6] = true,
		[16] = true,
		[21] = true,
		[26] = true,
		[31] = true,
		[36] = true,
		[135] = true,
		[140] = true,
		[145] = true,
		[150] = true,
	},
	["contestedAlliance"] = {
		[9] = true,
		[17] = true,
		[22] = true,
		[27] = true,
		[32] = true,
		[37] = true,
		[137] = true,
		[142] = true,
		[147] = true,
		[152] = true,
	},
	["contestedHorde"] = {
		[12] = true,
		[19] = true,
		[24] = true,
		[29] = true,
		[34] = true,
		[39] = true,
		[139] = true,
		[144] = true,
		[149] = true,
		[154] = true,
	},
	["controlledAlliance"] = {
		[11] = true,
		[18] = true,
		[23] = true,
		[28] = true,
		[33] = true,
		[38] = true,
		[136] = true,
		[141] = true,
		[146] = true,
		[151] = true,
	},
	["controlledHorde"] = {
		[10] = true,
		[20] = true,
		[25] = true,
		[30] = true,
		[35] = true,
		[40] = true,
		[138] = true,
		[143] = true,
		[148] = true,
		[153] = true,
	},
};

local basesByMap = {
	[529] = { -- Arathi Basin
		[1] = L["Stables"],
		[2] = L["Gold Mine"],
		[3] = L["Blacksmith"],
		[4] = L["Lumber Mill"],
		[5] = L["Farm"],
	},
	[1105] = { -- Deepwind Gorge
		[1] = L["Pandaren Mine"],
		[2] = L["Center Mine"],
		[3] = L["Goblin Mine"],
	},
	[566] = { -- Eye of the Storm
		[1] = L["Draenei Ruins"],
		[2] = L["Mage Tower"],
		[3] = L["Fel Reaver Ruins"],
		[4] = L["Blood Elf Tower"],	
	},
	[968] = { -- Eye of the Storm (Rated)
		[1] = L["Draenei Ruins"],
		[2] = L["Mage Tower"],
		[3] = L["Fel Reaver Ruins"],
		[4] = L["Blood Elf Tower"],	
	},
	[761] = { -- The Battle for Gilneas
		[1] = L["Lighthouse"],
		[2] = L["Waterworks"],
		[3] = L["Mines"],
	},
	[628] = { -- Isle of Conquest
		[1] = L["Alliance Keep"],
		[2] = L["Quarry"],
		[3] = L["Docks"],
		[4] = L["Workshop"],
		[5] = L["Hangar"],
		[6] = L["Refinery"],
		[7] = L["Horde Keep"],
	},
	--[30] = { -- Alterac Valley
	--},
};

local baseToIconByMap = {
	[529] = { -- Arathi Basin
		[L["Stables"]] = "Interface\\ICONS\\Garrison_Building_Stables",
		[L["Gold Mine"]] = "Interface\\AddOns\\BGLive\\Textures\\GoldMineIcon",
		[L["Blacksmith"]] = "Interface\\ICONS\\Trade_BlackSmithing",
		[L["Farm"]] = "Interface\\ICONS\\Garrison_Building_Barn",
		[L["Lumber Mill"]] = "Interface\\ICONS\\Garrison_Building_Lumbermill",
	},
	[1105] = { -- Deepwind Gorge
		[L["Pandaren Mine"]] = "Interface\\AddOns\\BGLive\\Textures\\PandarenMineIcon",
		[L["Center Mine"]] = "Interface\\AddOns\\BGLive\\Textures\\GoldMineIcon",
		[L["Goblin Mine"]] = "Interface\\AddOns\\BGLive\\Textures\\GoblinMineIcon",
	},
	[566] = { -- Eye of the Storm
		[L["Mage Tower"]] = "Interface\\ICONS\\Garrison_Building_MageTower",
		[L["Draenei Ruins"]] = "Interface\\ICONS\\Inv_Misc_Tournaments_banner_Draenei",
		[L["Fel Reaver Ruins"]] = "Interface\\ICONS\\Spell_Fire_FelFlameRing",
		[L["Blood Elf Tower"]] = "Interface\\ICONS\\Inv_Misc_Tournaments_banner_Bloodelf",
	},
	[968] = { -- Eye of the Storm (Rated)
		[L["Mage Tower"]] = "Interface\\ICONS\\Garrison_Building_MageTower",
		[L["Draenei Ruins"]] = "Interface\\ICONS\\Inv_Misc_Tournaments_banner_Draenei",
		[L["Fel Reaver Ruins"]] = "Interface\\ICONS\\Spell_Fire_FelFlameRing",
		[L["Blood Elf Tower"]] = "Interface\\ICONS\\Inv_Misc_Tournaments_banner_Bloodelf",
	},
	[761] = { -- The Battle for Gilneas
		[L["Lighthouse"]] = "Interface\\ICONS\\Achievement_Battleground_BattleForGilneas",
		[L["Waterworks"]] = "Interface\\ICONS\\INV_Elemental_Primal_Water",
		[L["Mines"]] = "Interface\\AddOns\\BGLive\\Textures\\GoldMineIcon",
	},
	[628] = { -- Isle of Conquest
		[L["Alliance Keep"]] = "Interface\\AddOns\\BGLive\\Textures\\AllianceTowerIcon",
		[L["Quarry"]] = "Interface\\ICONS\\INV_Stone_10",
		[L["Docks"]] = "Interface\\ICONS\\inv_helmet_66",
		[L["Workshop"]] = "Interface\\ICONS\\Garrison_Building_Workshop",
		[L["Hangar"]] = "Interface\\ICONS\\Achievement_Dungeon_HordeAirship",
		[L["Refinery"]] = "Interface\\ICONS\\Ability_Vehicle_OilJets",
		[L["Horde Keep"]] = "Interface\\AddOns\\BGLive\\Textures\\HordeTowerIcon",
	},
};

local baseToFactionNumber = {};

local contestedBases = {};


--[[
****************************************
****** HANDLER METHODS START HERE ******
****************************************
]]
function BaseIndicator:ConstructObject(indicator, addonName, frameGroup)
	
	ArenaLive:CopyClassMethods(BaseIndicatorClass, indicator);
	
	indicator.addon = addonName;
	indicator.group = frameGroup;
	indicator.numIcons = 0;
	
	for i = 1, NUM_MAX_BASES do
		BaseIndicator:ConstructIcon(indicator);
	end
	
	baseIndicators[indicator] = true;
	
end
function BaseIndicator:ConstructIcon(indicator)
	local prefix = indicator:GetName();
	local database = ArenaLive:GetDBComponent(indicator.addon, self.name, indicator.group);
	local xOffset = database.IconXOffset;
	local yOffset = database.IconYOffset;
	local direction = database.Direction;
	
	local point, relativePoint, relativeTo;
	if ( direction == "UP" ) then
		point = "BOTTOM";
		relativePoint = "TOP";
	elseif ( direction == "RIGHT" ) then
		point = "LEFT";
		relativePoint = "RIGHT";
	elseif ( direction == "DOWN" ) then
		point = "TOP";
		relativePoint = "BOTTOM";
		yOffset = -yOffset;
	elseif ( direction == "LEFT" ) then
		point = "RIGHT";
		relativePoint = "LEFT";
		xOffset = -xOffset;
	end
	
	local iconNum = indicator.numIcons + 1;
	if ( iconNum == 1 ) then
		relativePoint = point;
		relativeTo = indicator;
		xOffset = 0;
		yOffset = 0;
	else
		relativeTo = indicator["icon"..indicator.numIcons];
	end
	
	local name;
	if ( prefix ) then
		name = prefix.."Icon"..iconNum;
	end
	
	local icon = CreateFrame("Frame", name, indicator, "BGLiveBaseIconTemplate");
	icon:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset);
	
	Cooldown:ConstructObject(icon.cooldown, indicator.addon);
	indicator["icon"..iconNum] = icon;
	indicator.numIcons = iconNum;
end

function BaseIndicator:Enable(battlefieldMapID, battlefieldStatusID)
	self.currentStatusID = battlefieldStatusID;
	self.currentMap = battlefieldMapID;
	
	BaseIndicator:RegisterEvent("WORLD_MAP_UPDATE");
	BaseIndicator:UpdateAvailableBases();
	BaseIndicator:QueryLandmarks();
	self.enabled = true;
end

function BaseIndicator:EnableAll()
	for indicator in pairs(baseIndicators) do
		indicator:Enable();
	end
end

function BaseIndicator:Disable()
	BaseIndicator:UnregisterEvent("WORLD_MAP_UPDATE");
	self.currentMap = nil;
	table.wipe(baseToFactionNumber);
	self.enabled = false;
end

function BaseIndicator:DisableAll()
	for indicator in pairs(baseIndicators) do
		indicator:Disable();
	end
end

function BaseIndicator:UpdateAll()
	for indicator in pairs(baseIndicators) do
		indicator:Update();
	end
end

function BaseIndicator:UpdateAvailableBases()
	if ( self.currentMap and basesByMap[self.currentMap] ) then
		for id, baseName in pairs(basesByMap[self.currentMap]) do
			baseToFactionNumber[baseName] = -1;
		end
	else
		table.wipe(baseToFactionNumber);
	end
end

function BaseIndicator:QueryCapturePatterns(msg)
	local faction, location, action;
	for pattern, actionType in pairs(captureStringsToAction) do
		faction, location = string.match(msg, pattern);
		if ( faction and location ) then
			action = actionType;
			break;
		end
	end
	
	return faction, location, action;
end

function BaseIndicator:GetStateByTextureID(textureID)
	if ( not self.currentMap ) then
		return nil, nil;
	end
	
	if (  baseStates["neutral"][textureID] ) then
		return -1, false;
	elseif ( baseStates["contestedAlliance"][textureID] ) then
		return 1, true;
	elseif ( baseStates["contestedHorde"][textureID] ) then -- Contested (Horde)
		return 0, true;
	elseif ( baseStates["controlledAlliance"][textureID] ) then -- Controlled by the Alliance
		return 1, false;
	elseif ( baseStates["controlledHorde"][textureID] ) then -- Controlled by the Horde
		return 0, false;
	else
		return nil, nil;
	end
end

function BaseIndicator:QueryLandmarks()
	if ( not self.enabled ) then
		return;
	end
	
	local wasChangeMade;
	for i = 1, GetNumMapLandmarks() do
		local location, description, textureID = GetMapLandmarkInfo(i);
		if ( baseToFactionNumber[location] and location and textureID ) then
			local factionNumber, isContested = BaseIndicator:GetStateByTextureID(textureID);
			if ( BaseIndicator:HasBaseStateChanged(location, factionNumber, isContested) ) then
				wasChangeMade = true;
				BaseIndicator:SetBaseOwner(location, factionNumber, isContested);
			end
		end
	end
	
	if ( wasChangeMade ) then
		self:UpdateAll();
	end
end

function BaseIndicator:HasBaseStateChanged(location, factionNumber, isContested)
	if ( factionNumber == baseToFactionNumber[location] and ( ( isContested and contestedBases[location] ) or ( not isContested and not contestedBases[location] ) ) ) then
		return false;
	else
		return true;
	end
end

function BaseIndicator:GetBaseInfo(location)
	if ( self.currentMap and baseToFactionNumber[location] ) then
		return baseToIconByMap[self.currentMap][location], baseToFactionNumber[location], contestedBases[location];
	else
		return nil, nil, nil;
	end
end

function BaseIndicator:SetBaseOwner(location, factionNumber, isContested)
	if ( isContested ) then
		contestedBases[location] = GetTime() + SEC_TO_TURN_BASE;
	else
		contestedBases[location] = nil;
	end
	
	baseToFactionNumber[location] = factionNumber;
end

function BaseIndicator:OnEvent(event, ...)
	local filter = ...;
	if ( event == "UPDATE_BATTLEFIELD_STATUS" ) then
		local isSpectator = IsSpectator();
		local status = GetBattlefieldStatus(filter);
		local mapID = select(8, GetInstanceInfo());
		if ( status == "active" and isSpectator and basesByMap[mapID] ) then
			self:Enable(mapID, filter);
			self:EnableAll();
			self:UpdateAvailableBases();
			self:UpdateAll();
		elseif ( status == "none" and filter == self.currentStatusID ) then
			self:Disable();
			self:DisableAll();
		end
	elseif ( event == "WORLD_MAP_UPDATE" ) then
		BaseIndicator:QueryLandmarks();
	end
end

-- Initial disable:
BaseIndicator:Disable();

--[[
****************************************
******* CLASS METHODS START HERE *******
****************************************
]]--
function BaseIndicatorClass:Enable()
	self:Show();
	self.enabled = true;
	self:Update();
end

function BaseIndicatorClass:Disable()
	self:Hide();
	self:Reset();
	self.enabled = false;
end

function BaseIndicatorClass:Update()
	if ( not self.enabled ) then
		return;
	end
	
	if ( BaseIndicator.enabled and BaseIndicator.currentMap ) then
		local database = ArenaLive:GetDBComponent(self.addon, BaseIndicator.name, self.group);
		local addonDB = ArenaLive:GetDBComponent(self.addon);
		local numBases = #basesByMap[BaseIndicator.currentMap];
		local theTime = GetTime();
		for i = 1, NUM_MAX_BASES do
			local icon = self["icon"..i];
			
			if ( i <= numBases ) then
				local location = basesByMap[BaseIndicator.currentMap][i];
				local baseTexture, factionNumber, contestedExpires = BaseIndicator:GetBaseInfo(location);
				icon.texture:SetTexture(baseTexture);
				
				local bRed, bGreen, bBlue, bAlpha = 1, 1, 1, 1;
				local sRed, sGreen, sBlue, sAlpha = 0, 0, 0, 1;
				if ( contestedExpires and contestedExpires > theTime ) then
					if ( factionNumber == 0 ) then
						sRed, sGreen, sBlue = unpack(addonDB.TeamB.Colour);
					elseif ( factionNumber == 1 ) then
						sRed, sGreen, sBlue = unpack(addonDB.TeamA.Colour);
					end
				elseif ( factionNumber == 0 ) then
					bRed, bGreen, bBlue, bAlpha = unpack(addonDB.TeamB.Colour);
				elseif ( factionNumber == 1 ) then
					bRed, bGreen, bBlue, bAlpha = unpack(addonDB.TeamA.Colour);
				end
				
				if ( not contestedExpires ) then
					icon.border:SetVertexColor(bRed, bGreen, bBlue, bAlpha);
				end
				
				-- Set Cooldown:
				local startTime;
				if ( contestedExpires and contestedExpires > theTime ) then
					startTime = contestedExpires - SEC_TO_TURN_BASE;
					icon.cooldown:SetSwipeColor(sRed, sGreen, sBlue, sAlpha);
					icon.cooldown:Set(startTime, SEC_TO_TURN_BASE);
				else
					icon.cooldown:Reset();
				end
				
				icon:Show();
			else
				icon:Hide();
			end
		end
		
		local xOffset, yOffset = database.IconXOffset, database.IconYOffset;
		local width, height;
		if ( database.Direction == "UP" or database.Direction == "DOWN" ) then
			width = BASE_ICON_WIDTH + xOffset * ( numBases - 1 );
			height = BASE_ICON_HEIGHT * numBases + yOffset *  ( numBases - 1 );
		elseif ( database.Direction == "RIGHT" or database.Direction == "LEFT" ) then
			width = BASE_ICON_WIDTH * numBases + xOffset * ( numBases - 1 );
			height = BASE_ICON_HEIGHT + yOffset * ( numBases - 1 );
		end
		self:SetSize(width, height);
	else
		self:Reset();
	end
end

function BaseIndicatorClass:Reset()
	for i = 1, self.numIcons do
		local icon = self["icon"..i];
		icon.texture:SetTexture();
		icon.cooldown:Reset();
		icon:Hide();
	end
end