--[[
    BGLive is an user interface for spectated battleground 
	wargames in World of Warcraft.
    Copyright (C) 2015  Harald Böhm <harald@boehm.agency>
	Further contributors: Jochen Taeschner and Romina Schmidt.
	
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local addonName, L = ...;

--[[
MAP IDS:
Blade's Edge Arena = 562
Dalaran Arena = 617
Nagrand Arena = 559
Ruins of Lordaeron = 572
The Ring of Valor = 618
The Tiger's Peak = 1134
Tol'Viron Arena = 980

Alterac Valley = 30
Arathi Basin = 529
Deepwind Gorge = 1105
Eye of the Storm = 566
Eye of the Storm (Rated) = 968
Isle of Conquest = 628
Silvershard Mines = 727
Strand of the Ancients = 607
Temple of Kotmogu = 998
The Battle for Gilneas = 761
Twin Peaks = 726
Warsong Gulch = 489
]]
BGLive.defaults = {
	["BaseIndicator"] = {
		["Direction"] = "RIGHT",
		["IconXOffset"] = 25,
		["IconYOffset"] = 0
	},
	["Cooldown"] =	{
		["ShowText"] = true,
		["StaticSize"] = false,
		["TextSize"] = 8,
	},
	["CCIndicator"] =	{
		["Priorities"] = {
			["defCD"] = 6,
			["offCD"] = 5,
			["stun"] = 4,
			["silence"] = 3,
			["crowdControl"] = 2,
			["root"] = 1,
			["usefulBuffs"] = 0,
			["usefulDebuffs"] = 0,
		},
	},
	["FirstLogin"] = true,
	["FollowTarget"] = true,
	["FrameMover"] = {
		["FrameLock"] = true,
	},
	["ShowScoreBoard"] = true,
	["SideFrames"] = {
		["UnitFrame"] = {
			["Enabled"] = true,
			["TooltipMode"] = "Never",
			["Scale"] = 1,
		},
		["CCIndicator"] = {
			["Enabled"] = true,
		},
		["HealthBar"] = {
			["ColourMode"] = "team",
			["ShowHealPrediction"] = true,
			["ShowAbsorb"] = true,
		},
		["HealthBarText"] = {
			["BarText"] = "%CURR_SHORT% (%PERCENT_SHORT%%)",
			["FontObject"] = "BGLiveSideFrameStatusBarFont",
			["ShowDeadOrGhost"] = true,
			["ShowDisconnect"] = true,
		},
		["PowerBar"] = {
		},
		["PowerBarText"] = {
			["BarText"] = "%CURR_SHORT% (%PERCENT_SHORT%%)",
			["FontObject"] = "BGLiveSideFrameStatusBarFont",
		},
		["Portrait"] = {
			["Type"] = "class",
		},
		["Icon"] = {
			[1] = {
				["Type"] = "trinket",
			},
			[2] = {
				["Type"] = "defensiveCD",
			},
		},
		["NameText"] = {
			["ColourMode"] = "none",
			["FontObject"] = "BGLiveSideFrameNameFont",
		},
	},
	["TeamA"] = {
		["Colour"] = { 0, 0.48, 0.87, 1 },
	},
	["TeamB"] = {
		["Colour"] = { 0.87, 0, 0, 1 },
	},
	["Version"] = "3.2.0b",
};

local baseToFaction = {};

function BGLive:Enable(battlefieldStatusID, mapID)
	if ( self.enabled and battlefieldStatusID == self.currentID ) then
		return;
	end

	-- Register needed events at this point.
	BGLive:RegisterEvent("ARENA_OPPONENT_UPDATE");
	BGLive:RegisterEvent("AL_SPEC_PLAYER_UPDATE");
	BGLive:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE");
	BGLive:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE");
	BGLive:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
	BGLive:RegisterEvent("CHAT_MSG_RAID_BOSS_EMOTE");
	BGLive:RegisterEvent("PLAYER_TARGET_CHANGED");
	BGLive:RegisterEvent("BATTLEGROUND_POINTS_UPDATE");
	BGLive:RegisterEvent("WORLD_STATE_UI_TIMER_UPDATE");
	if ( mapID == 30 or mapID == 628 or mapID == 998 or mapID == 727 ) then
		BGLive:RegisterEvent("UPDATE_WORLD_STATES");
	end

	self.currentMap = mapID;
	self.currentID = battlefieldStatusID;
	
	-- Update Leader to check for team swap:
	local leaderName, leaderRealm = ArenaLiveSpectator:RequestLeader();
	if ( leaderName and leaderRealm ) then
		ArenaLiveSpectator:SetLeader(leaderName, leaderRealm);
	end	
	
	self:EnableCarrierFrames();
	self:EnableSideFrames();
		
	self:UpdateCarrierFrames();
	self:UpdateSideFrames();
		
	BGLiveScoreBoard:UpdatePoints();
	BGLiveScoreBoard:UpdateTimer();
	ArenaLiveSpectatorHideUIButton:Show();
	
	BGLiveScoreBoard:UpdateTeamName("TeamA");
	BGLiveScoreBoard:UpdateTeamName("TeamB");
	BGLiveScoreBoard:UpdateTeamScore("TeamA");
	BGLiveScoreBoard:UpdateTeamScore("TeamB");	
	
	self:Show();
	UIParent:Hide();
	self.enabled = true;
end

function BGLive:Disable()
	BGLive:DisableSideFrames();
	BGLive:DisableCarrierFrames();
	BGLiveScoreBoard:Reset();
	
	if ( not ArenaLiveSpectator:IsEnabled() ) then
		ArenaLiveSpectatorHideUIButton:Hide();
	end

	-- Unregister events:
	BGLive:UnregisterEvent("AL_SPEC_PLAYER_UPDATE");
	BGLive:UnregisterEvent("ARENA_OPPONENT_UPDATE");
	BGLive:UnregisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE");
	BGLive:UnregisterEvent("CHAT_MSG_BG_SYSTEM_HORDE");
	BGLive:UnregisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
	BGLive:UnregisterEvent("CHAT_MSG_RAID_BOSS_EMOTE");
	BGLive:UnregisterEvent("PLAYER_TARGET_CHANGED");
	BGLive:UnregisterEvent("BATTLEGROUND_POINTS_UPDATE");
	BGLive:UnregisterEvent("WORLD_STATE_UI_TIMER_UPDATE");	
	BGLive:UnregisterEvent("UPDATE_WORLD_STATES");
		
	
	self.currentMap = nil;
	ArenaLiveSpectator.leader = nil;
	self.swapTeams = false;
	self.worldStateTimerIndex = nil;
	self.enabled = false;
	UIParent:Show();
	self:Hide();
end

function BGLive:GetCurrentMapID()
	return self.currentMap;
end

function BGLive:OnEvent (event, ...)
	local filter = ...;
	if ( event == "ADDON_LOADED" and filter == addonName ) then
		self:InitialiseSideFrames();
		self:InitialiseCarrierFrames();
		BGLiveScoreBoard:Initialise();
		BGLiveChatMessageFrame:Initialise();
		ArenaLive:ConstructHandlerObject(BGLiveBaseIndicator, "BaseIndicator", addonName);
		BGLive:Disable();
	elseif ( event == "ARENA_OPPONENT_UPDATE" ) then
		self:UpdateCarrierFrames();
	elseif ( event == "CHAT_MSG_BG_SYSTEM_ALLIANCE" or event == "CHAT_MSG_BG_SYSTEM_HORDE" or event == "CHAT_MSG_BG_SYSTEM_NEUTRAL" or event == "CHAT_MSG_RAID_BOSS_EMOTE" ) then
		BGLiveChatMessageFrame:OnEvent(event, ...);
		if ( event == "CHAT_MSG_BG_SYSTEM_NEUTRAL" and filter == L["The battle has begun!"] ) then
			BGLive:UpdateSideFrames();
		end
	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
		if ( self.enabled ) then
			local database = ArenaLive:GetDBComponent(ArenaLiveSpectator.name);
			if ( database.FollowTarget ) then
				C_Commentator.FollowUnit("target");
			end
		end
	elseif ( event == "AL_SPEC_PLAYER_UPDATE" ) then
		local unit, state = ...;
		BGLive:UpdateSideFrameByUnit(unit, state);
	elseif ( event == "BATTLEGROUND_POINTS_UPDATE" ) then
		BGLiveScoreBoard:UpdatePoints();
	elseif ( event == "UPDATE_BATTLEFIELD_STATUS" ) then
		local status, mapID = GetBattlefieldStatus(filter);
		local isSpectator, gameType = IsSpectator();
		local winner = GetBattlefieldWinner();
		if ( status == "active" and filter == self.currentID and winner ) then
			local database = ArenaLive:GetDBComponent(ArenaLiveSpectator.name);
			if ( ( not self.swapTeams and winner == 1 ) or ( self.swapTeams and winner == 0 ) ) then
				database.TeamA.Score = database.TeamA.Score + 1;
			elseif ( ( not self.swapTeams and winner == 0 ) or ( self.swapTeams and winner == 1 ) ) then
				database.TeamB.Score = database.TeamB.Score + 1;
			elseif ( winner == 255 ) then
				database.TeamB.Score = database.TeamB.Score + 1;
				database.TeamA.Score = database.TeamA.Score + 1;
			end
			
			BGLiveScoreBoard:UpdateTeamScore("TeamA");
			BGLiveScoreBoard:UpdateTeamScore("TeamB");
		elseif ( status == "active" and isSpectator and gameType == "battleground" ) then
			local mapID = select(8, GetInstanceInfo());
			self:Enable(filter, mapID);
		elseif (  status == "none" and filter == self.currentID ) then
			self:Disable();
		end
	elseif ( event == "WORLD_STATE_UI_TIMER_UPDATE" ) then
		if ( not self.worldStateTimerIndex ) then
			for index = 1, GetNumWorldStateUI() do
				local uiType = GetWorldStateUIInfo(index);
				if ( uiType == 3 ) then -- 3 seems to be the identifier for timers.
					self.worldStateTimerIndex = index;
					break;
				end
			end
		end
		BGLiveScoreBoard:UpdateTimer();
	elseif ( event == "UPDATE_WORLD_STATES" ) then
		BGLiveScoreBoard:UpdatePoints();
	end
end

function BGLive:UpdateDB()
	local database = ArenaLive:GetDBComponent(addonName);
	if ( not database.Version or database.Version == "0.0.1a" or database.Version == "3.1.0b" ) then
		database.Version = "3.1.1b";
		database.SideFrames.HealthBar.ColourMode = "class";
	end
	
	if ( database.Version ~= "3.2.0b" ) then
		database.Version = "3.2.0b";
	end
end

ArenaLive:ConstructAddon(BGLive, addonName, true, BGLive.defaults, false, "BGLIVE_DATABASE");
BGLive:RegisterEvent("ADDON_LOADED");
BGLive:RegisterEvent("UPDATE_BATTLEFIELD_STATUS");
